@@ -1,20 +1,9 @@
-/* This file is part of the KDE Project
-   Copyright (c) 2007-2011 Sebastian Trueg <trueg@kde.org>
-   Copyright (c) 2012-2014 Vishesh Handa <me@vhanda.in>
+/*
+    This file is part of the KDE Project
+    SPDX-FileCopyrightText: 2007-2011 Sebastian Trueg <trueg@kde.org>
+    SPDX-FileCopyrightText: 2012-2014 Vishesh Handa <me@vhanda.in>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "filewatch.h"
@@ -23,16 +12,13 @@
 #include "pendingfilequeue.h"
 #include "regexpcache.h"
 #include "database.h"
-#include "pendingfile.h"
 #include "baloodebug.h"
 
 #include "kinotify.h"
 
-#include <QDir>
 #include <QDateTime>
-#include <QDBusConnection>
-
-#include <syslog.h>
+#include <QFileInfo>
+#include <QDir>
 
 using namespace Baloo;
 
@@ -40,7 +26,7 @@ FileWatch::FileWatch(Database* db, FileIndexerConfig* config, QObject* parent)
     : QObject(parent)
     , m_db(db)
     , m_config(config)
-    , m_dirWatch(0)
+    , m_dirWatch(nullptr)
 {
     Q_ASSERT(db);
     Q_ASSERT(config);
@@ -68,45 +54,46 @@ FileWatch::FileWatch(Database* db, FileIndexerConfig* config, QObject* parent)
     connect(m_dirWatch, &KInotify::installedWatches, this, &FileWatch::installedWatches);
 }
 
-
 FileWatch::~FileWatch()
 {
 }
 
-void FileWatch::watchIndexedFolders()
-{
-    // Watch all indexed folders
-    QStringList folders = m_config->includeFolders();
-    Q_FOREACH (const QString& folder, folders) {
-        watchFolder(folder);
-    }
-}
-
 // FIXME: listen to Create for folders!
 void FileWatch::watchFolder(const QString& path)
 {
-    qCDebug(BALOO) << path;
     if (m_dirWatch && !m_dirWatch->watchingPath(path)) {
         KInotify::WatchEvents flags(KInotify::EventMove | KInotify::EventDelete | KInotify::EventDeleteSelf
                                     | KInotify::EventCloseWrite | KInotify::EventCreate
                                     | KInotify::EventAttributeChange | KInotify::EventModify);
 
-        m_dirWatch->addWatch(path, flags, KInotify::WatchFlags());
+        m_dirWatch->addWatch(path, flags, KInotify::FlagOnlyDir);
     }
 }
 
 void FileWatch::slotFileMoved(const QString& urlFrom, const QString& urlTo)
 {
-    m_metadataMover->moveFileMetadata(urlFrom, urlTo);
-}
+    if (m_config->shouldBeIndexed(urlTo)) {
+        m_metadataMover->moveFileMetadata(urlFrom, urlTo);
+    } else {
+        QFileInfo src(urlFrom);
+        QString url = urlFrom;
 
+        if (src.isDir() && !url.endsWith(QLatin1Char('/'))) {
+            url.append(QLatin1Char('/'));
+        }
+
+        PendingFile file(url);
+        file.setDeleted();
+
+        m_pendingFileQueue->enqueue(file);
+    }
+}
 
 void FileWatch::slotFileDeleted(const QString& urlString, bool isDir)
 {
     // Directories must always end with a trailing slash '/'
     QString url = urlString;
-    if (isDir) {
-        Q_ASSERT(!url.endsWith('/'));
+    if (isDir && !url.endsWith(QLatin1Char('/'))) {
         url.append(QLatin1Char('/'));
     }
 
@@ -116,7 +103,6 @@ void FileWatch::slotFileDeleted(const QString& urlString, bool isDir)
     m_pendingFileQueue->enqueue(file);
 }
 
-
 void FileWatch::slotFileCreated(const QString& path, bool isDir)
 {
     Q_UNUSED(isDir);
@@ -166,15 +152,15 @@ void FileWatch::slotAttributeChanged(const QString& path)
 // inotify_add_watch fails with ENOSPC.
 void FileWatch::slotInotifyWatchUserLimitReached(const QString&)
 {
-    //If we got here, we hit the limit and couldn't authenticate to raise it,
-    // so put something in the syslog so someone notices.
-    syslog(LOG_USER | LOG_WARNING, "KDE Baloo File Indexer has reached the inotify folder watch limit. File changes will be ignored.");
+    // If we got here, we hit the limit and are not allowed to raise it,
+    // so put something in the log.
+    qCWarning(BALOO) << "KDE Baloo File Indexer has reached the inotify folder watch limit. File changes will be ignored.";
     // we do it the brutal way for now hoping with new kernels and defaults this will never happen
     // Delete the KInotify
     // FIXME: Maybe we should be aborting?
     if (m_dirWatch) {
         m_dirWatch->deleteLater();
-        m_dirWatch = 0;
+        m_dirWatch = nullptr;
     }
     Q_EMIT installedWatches();
 }
@@ -182,11 +168,37 @@ void FileWatch::slotInotifyWatchUserLimitReached(const QString&)
 void FileWatch::updateIndexedFoldersWatches()
 {
     if (m_dirWatch) {
-        QStringList folders = m_config->includeFolders();
-        Q_FOREACH (const QString& folder, folders) {
-            m_dirWatch->removeWatch(folder);
-            watchFolder(folder);
+        const QStringList excludedFolders = m_config->excludeFolders();
+        const QStringList includedFolders = m_config->includeFolders();
+
+        for (const QString& folder : excludedFolders) {
+            // Remove watch for new excluded folders
+            if (!m_excludedFolders.contains(folder)) {
+                m_dirWatch->removeWatch(folder);
+            }
         }
+        for (const QString& folder : m_excludedFolders) {
+            // Add no longer excluded folders
+            if (!excludedFolders.contains(folder)) {
+                watchFolder(folder);
+            }
+        }
+
+        for (const QString& folder : m_includedFolders) {
+            // Remove no longer included folders
+            if (!includedFolders.contains(folder)) {
+                m_dirWatch->removeWatch(folder);
+            }
+        }
+        for (const QString& folder : includedFolders) {
+            if (!m_includedFolders.contains(folder)) {
+                watchFolder(folder);
+            }
+        }
+
+        m_excludedFolders = excludedFolders;
+        m_includedFolders = includedFolders;
     }
 }
 
+#include "moc_filewatch.cpp"
